/**
 * Copyright (c) 2000-2011 Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.portal.workflow.kaleo;

import com.liferay.portal.kernel.exception.PortalException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.workflow.DefaultWorkflowTask;
import com.liferay.portal.kernel.workflow.WorkflowTaskAssignee;
import com.liferay.portal.workflow.kaleo.model.KaleoInstance;
import com.liferay.portal.workflow.kaleo.model.KaleoInstanceToken;
import com.liferay.portal.workflow.kaleo.model.KaleoTaskInstanceToken;
import com.liferay.portal.workflow.kaleo.util.KaleoTaskAssignmentInstanceUtil;
import com.liferay.portal.workflow.kaleo.util.WorkflowContextUtil;

import java.io.Serializable;

import java.util.List;
import java.util.Map;

/**
 * @author Michael C. Han
 */
public class WorkflowTaskAdapter extends DefaultWorkflowTask {

	public WorkflowTaskAdapter(
			KaleoTaskInstanceToken kaleoTaskInstanceToken,
			Map<String, Serializable> workflowContext)
		throws PortalException, SystemException {

		_kaleoTaskInstanceToken = kaleoTaskInstanceToken;

		setCreateDate(kaleoTaskInstanceToken.getCreateDate());
		setCompletionDate(kaleoTaskInstanceToken.getCompletionDate());
		setDescription(kaleoTaskInstanceToken.getKaleoTask().getDescription());
		setDueDate(kaleoTaskInstanceToken.getDueDate());
		setName(kaleoTaskInstanceToken.getKaleoTask().getName());

		if (workflowContext != null) {
			setOptionalAttributes(workflowContext);
		}
		else {
			setOptionalAttributes(
				WorkflowContextUtil.convert(
					kaleoTaskInstanceToken.getWorkflowContext()));
		}

		KaleoInstanceToken kaleoInstanceToken =
			kaleoTaskInstanceToken.getKaleoInstanceToken();
		KaleoInstance kaleoInstance = kaleoInstanceToken.getKaleoInstance();

		setWorkflowDefinitionId(kaleoInstance.getKaleoDefinitionId());
		setWorkflowDefinitionName(kaleoInstance.getKaleoDefinitionName());
		setWorkflowDefinitionVersion(kaleoInstance.getKaleoDefinitionVersion());
		setWorkflowInstanceId(kaleoInstance.getKaleoInstanceId());

		List<WorkflowTaskAssignee> workflowTaskAssignees =
			KaleoTaskAssignmentInstanceUtil.getWorkflowTaskAssignees(
				kaleoTaskInstanceToken);

		setWorkflowTaskAssignees(workflowTaskAssignees);

		setWorkflowTaskId(kaleoTaskInstanceToken.getKaleoTaskInstanceTokenId());
	}

	public KaleoTaskInstanceToken getKaleoTaskInstanceToken() {
		return _kaleoTaskInstanceToken;
	}

	private KaleoTaskInstanceToken _kaleoTaskInstanceToken;

}