/**
 * Copyright (c) 2000-2011 Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.portal.workflow.kaleo.runtime.node;

import com.liferay.portal.kernel.workflow.WorkflowException;
import com.liferay.portal.workflow.kaleo.definition.NodeType;

import java.util.HashMap;
import java.util.Map;

/**
 * @author Michael C. Han
 */
public class NodeExecutorFactory {

	public static NodeExecutor getNodeExecutor(NodeType nodeType)
		throws WorkflowException {

		NodeExecutor nodeExecutor = _nodeExecutors.get(nodeType);

		if (nodeExecutor == null) {
			throw new WorkflowException("Invalid node type " + nodeType);
		}

		return nodeExecutor;
	}

	public void setNodeExectors(Map<String, NodeExecutor> nodeExecutors) {
		_nodeExecutors = new HashMap<NodeType, NodeExecutor>();

		for (Map.Entry<String, NodeExecutor> entry : nodeExecutors.entrySet()) {
			NodeType nodeType = NodeType.valueOf(entry.getKey());

			_nodeExecutors.put(nodeType, entry.getValue());
		}
	}

	public static Map<NodeType, NodeExecutor> _nodeExecutors;

}